/*
                           Copyright (C) 1995 
             John R.Callahan, Sudhakar Ramakrishnan, Sun Wei.
	         Web Integrated Software Project 
                   Management and Metrics Tool
                           (WISE)

WISE [both binary and source] (hereafter, Software) is 
copyrighted by John R.Callahan, Sudhakar Ramakrishnan , Sun Wei and 
ownership remains with the authors.

The authors grant you (hereafter, Licensee), a license to use the Software
for academic, research and internal business purposes only, without a
fee.  Licensee may distribute the binary to third parties providing 
that the copyright notice and this statement appears on all copies and
that no charge is associated with such copies. 

Licensee may make derivative works.  However, if Licensee distributes
any derivative work based on or derived from the Software, then
Licensee will (1) notify the authors regarding its distribution of 
the derivative work, and (2) clearly notify users that such derivative 
work is a modified version and not the original WISE tool distributed 
by the authors.

Any Licensee wishing to make commercial use of the Software should
contact the authors to negotiate an appropriate license for such
commercial use.  Commercial use includes (1) integration of all or
part of the source code into a product for sale or license by or on
behalf of licensee to third parties, or (2) distribution of the binary
code or source code to third parties that need it to utilize a
commercial product sold or licensed by or on behalf of Licensee.

THE AUTHORS MAKE NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS 
SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR 
IMPLIED WARRANTY.  THE AUTHORS SHALL NOT BE LIABLE FOR ANY DAMAGES
SUFFERED BY USERS OF THIS SOFTWARE.

By using or copying this Software, Licensee agrees to abide by the
copyright law and all other applicable laws of the U.S., including but
not limited, to, export control laws, and the terms of this license.
The authors shall have the right to terminate this license immediately by
written notice upon Licensee's breach of, or non-compliance with, any
of its terms.  Licensee may be held legally responsible for any
copyright infringement that is caused, directly or indirectly, by
Licensee's failure to abide by the terms of this license.

Direct notices to:
	
	John R.Callahan 
	886 Chestnut Ridge Road
	Concurrent Engineering Research Center
	West Virginia University
	Morgantown, WV 26505
	USA

	phone:	(304) 293-7226
	email:	callahan@cerc.wvu.edu
*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "msql.h"
#include "cgi.h"
#include "wise.h"

/*  Note that under gnu libc, a null passed to the string
 * library routines means "operate on string located at address
 * zero", which is not what most programmers expect to happen.
 * in particular, the code below expects any non-null string to
 * match the null string. Thus, we wrap and redefine strcmp.
 */
int linStrCmp (const char *s1, const char *s2) {
   if (0x0 == s1) return 0;
   if (0x0 == s2) return 0;
   return (strcmp(s1,s2));
}
   
#define strcmp(a,b) linStrCmp(a,b)

void
wise_printselectoptions(char* values)
{
  int i;

  if(values == NULL) return;
  printf("<OPTION>");
  for(i=0;values[i] != '\0';i++) {
    if(values[i] == WISESEPCHAR) {
      i++;
      printf("<OPTION>");
    }
    printf("%c",values[i]);
  }
}

void
wise_printradioitems(char* field,char* values,char* current)
{
  int i,j,k,first=1;

  if(values == NULL) return;
  printf("<INPUT TYPE=radio NAME=%s VALUE=\"",field);
  j = 0;
  for(i=0;values[i] != '\0';i++) {
    if(values[i] == WISESEPCHAR) {
      if(current != NULL && strncmp(current,values+j,strlen(current)) == 0) {
	printf("\" checked>");
      } else if(current == NULL) {
	if(first) { printf("\" checked>"); first=0; }
	else printf("\">");
      } else {
	printf("\">");
      }
      for(k=j;k < i;k++) {
	printf("%c",values[k]);
      }
      j = ++i;
      printf("<INPUT TYPE=radio NAME=%s VALUE=\"",field);
    }
    printf("%c",values[i]);
  }
  if(current != NULL && strncmp(current,values+j,strlen(current)) == 0) {
    printf("\" checked>");
  } else if(current == NULL) {
    if(first) { printf("\" checked>"); first=0; }
    else printf("\">");
  } else {
    printf("\">");
  }
  for(k=j;k < i;k++) {
    printf("%c",values[k]);
  }
}

void
wise_printcheckboxitems(char* field,char* values,char* current)
{
  char tbuf[WISEBUFMAX];
  int i,j,k;

  if(values == NULL) return;
  strcpy(tbuf,values);
  printf("<INPUT TYPE=checkbox NAME=%s VALUE=\"",field);
  j = 0;
  for(i=0;values[i] != '\0';i++) {
    if(values[i] == WISESEPCHAR) {
      tbuf[i] = '\0';
      if(current != NULL && strstr(current,tbuf+j) != NULL) {
	printf("\" checked>");
      } else {
	printf("\">");
      }
      for(k=j;k < i;k++) {
	printf("%c",values[k]);
      }
      j = ++i;
      printf("<INPUT TYPE=checkbox NAME=%s VALUE=\"",field);
    }
    printf("%c",values[i]);
  }
  if(current != NULL && strstr(current,tbuf+j) != NULL) {
    printf("\" checked>");
  } else {
    printf("\">");
  }
  for(k=j;k < i;k++) {
    printf("%c",values[k]);
  }
}

char*
wise_stringmacro(char* buf,char* ru)
{
  int i;
  time_t ts1970;
  struct tm *pstm;
  char dbuf[WISEBUFMAX];
  /* static char newbuf[WISEBUFMAX]; */
  char * newbuf = 0x0;

  /* Design notes:
   * (1) this routine leaks memory, but that's OK for a cgi bin.
   *     We will punt on this ugliness, since that's better than 
   *     the original static design.
   * (2) this function uses highly innefficient search techniques 
   *     and should be completely redesigned.
   */

  if (0x0 == buf) return "";
  if (0 == buf[0]) return "";

  newbuf = (char *) malloc (WISEBUFMAX);

  strcpy(newbuf,"");
  for(i=0;i < strlen(buf);i++) {
    if(strstr(buf+i,"$USER$") == buf+i) {
      if(WISEDEBUG) printf("%s is a prefix of %s","$USER$",buf+i);
      strcat(newbuf,ru);
      i = i + strlen("$USER$")-1;
    } else if(strstr(buf+i,"$REMOTE_HOST") == buf+i) {
      if(WISEDEBUG) printf("%s is a prefix of %s","$REMOTE_HOST$",buf+i);
      strcat(newbuf,getenv("REMOTE_HOST"));
      i = i + strlen("$REMOTE_HOST$")-1;
    } else if(strstr(buf+i,"$TODAY$") == buf+i) {
      if(WISEDEBUG) printf("%s is a prefix of %s","$TODAY$",buf+i);
      ts1970 = time(NULL);
      /* cftime(dbuf,"%Y-%m-%d",&ts1970); */
      pstm = localtime (&ts1970);
      strftime(dbuf,WISEBUFMAX,"%Y-%m-%d",pstm);
      strcat(newbuf,dbuf);
      i = i + strlen("$TODAY$")-1;
    } else {
      if(WISEDEBUG) printf(" NO!<BR>");
      strncat(newbuf,buf+i,1);
    }
  }
  if(WISEDEBUG) printf("newbuf = %s\n",newbuf);
  return newbuf;
}

void
wise_printhtml(char* buf,char* ru)
{
  int i,j;

  if(buf == NULL) return;
  if(strlen(buf) <= 0) return;
  j = 0;
  for(i=0;buf[i] != '\0';i++) {
    if(buf[i] == WISESEPCHAR) {
      buf[i] = '\0';
      j = i;
      break;
    }
  }
  if(j > 0) {
    printf("<A HREF=\"%s\">",buf+i+1);
    printf("%s</A>",buf);
  }
  buf[i] = WISESEPCHAR;
}

void
wise_printhtmlfield(char* buf,int fnum)
{
  int i,j;

  if(buf == NULL) return;
  if(strlen(buf) <= 0) return;
  j = 0;
  for(i=0;buf[i] != '\0';i++) {
    if(buf[i] == WISESEPCHAR) {
      buf[i] = '\0';
      j = i;
      break;
    }
  }
  if(j > 0) {
    if(fnum == 2) printf("%s",buf+i+1);
    else if(fnum == 1) printf("%s",buf);
  }
  buf[i] = WISESEPCHAR;
}

void
wise_printhtmlfields(char* field,char* buf,char* ru)
{
  int i;

  if(buf == NULL) return;
  if(strlen(buf) <= 0) return;
  for(i=0;buf[i] != '\0';i++) {
    if(buf[i] == WISESEPCHAR) {
      buf[i] = '\0';
      break;
    }
  }
  printf("<INPUT TYPE=hidden NAME=%shtml VALUE=\"%s\">",field,buf+i+1);
  printf("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">",field,buf);
  buf[i] = WISESEPCHAR;
}

char*
wise_getprojdb(char* projname)
{
  int i;

  for(i=0;wiseproject[i].projectname != NULL;i++) {
    if(strcmp(wiseproject[i].projectname,projname) == 0) return wiseproject[i].database;
  }
  return NULL;
}

char*
wise_getprojimage(char* projname)
{
  int i;

  for(i=0;wiseproject[i].projectname != NULL;i++) {
    if(strcmp(wiseproject[i].projectname,projname) == 0) return wiseproject[i].projectimage;
  }
  return NULL;
}

char*
wise_getprojhost(char* projname)
{
  int i;

  for(i=0;wiseproject[i].projectname != NULL;i++) {
    if(strcmp(wiseproject[i].projectname,projname) == 0) return wiseproject[i].dbhost;
  }
  return NULL;
}

void
wise_setprojport(char* projname)
{
  int i;
  char* buf;

  for(i=0;wiseproject[i].projectname != NULL;i++) {
    if(strcmp(wiseproject[i].projectname,projname) == 0) {
      buf = malloc(25);
      sprintf(buf,"MSQL_TCP_PORT=%d",wiseproject[i].dbport);
      putenv(buf);
    }
  }
  return;
}

char*
wise_getprojlabel(char* projname)
{
  int i;

  for(i=0;wiseproject[i].projectname != NULL;i++) {
    if(strcmp(wiseproject[i].projectname,projname) == 0) return wiseproject[i].label;
  }
  return NULL;
}

void
wise_replace(char* str,char find,char replacement,int len)
{
  int i;
  for(i=0;i < len;i++) {
    if(str[i] == find) str[i] = replacement;
  }
}

char*
wise_getaccess(char* ru,char* project)
{
  int i,j;
  static char buf[WISEBUFMAX];
  char* filterbuf;
  int started =0;

  strcpy(buf,"");
  if(WISEDEBUG) printf("getaccess: %s<BR>",ru);
  if(WISEDEBUG) printf("getaccess: %s<BR>",project);
  for(i=0;wiseuserroles[i].user != NULL;i++) {
    if(WISEDEBUG) printf("(1) does '%s' = '%s'?<BR>\n",ru,wiseuserroles[i].user);
    if(strcmp(ru,wiseuserroles[i].user) == 0) {
      for(j=0;wisetdlaccess[j].project != NULL;j++) {
	if(WISEDEBUG) printf("(2) does '%s' = '%s'?<BR>\n",wisetdlaccess[j].project,project);
	if(WISEDEBUG) printf("(2) does '%s' = '%s'?<BR>\n",wiseuserroles[i].role,wisetdlaccess[j].role);
	if((strcmp(wiseuserroles[i].role,wisetdlaccess[j].role)==0) && (strcmp(wisetdlaccess[j].project,project)==0)) {
	  if(wisetdlaccess[j].condition != NULL) {
	    if (started) strcat(buf," OR ");
            started = 1;
            strcat(buf,wisetdlaccess[j].condition);
          }
	}
      }
    }
  }
  filterbuf = wise_stringmacro(buf,ru);
  return filterbuf;
}

char*
wise_getroles(char* ru,char* project)
{
  int i,j;
  static char buf[WISEBUFMAX];

  strcpy(buf,"");
  for(i=0;wiseuserroles[i].user != NULL;i++) {
    if(strcmp(ru,wiseuserroles[i].user) == 0) {
      for(j=0;wisetdlaccess[j].project != NULL;j++) {
	if((strcmp(wiseuserroles[i].role,wisetdlaccess[j].role)==0) && (strcmp(wisetdlaccess[j].project,project)==0)) {
	  if(strlen(buf) > 0) strcat(buf,",");
	  strcat(buf,wiseuserroles[i].role);
	}
      }
    }
  }
  return buf;
}

WiseTDLField*
wise_getprojecttdl(char* project)
{
  int i;

  for(i=0;wiseproject[i].projectname != NULL;i++) {
    if(strcmp(wiseproject[i].projectname,project) == 0) return wiseproject[i].tdlfields;
  }
  return NULL;
}

void
wise_getprojects(char* ru,char* buf[])
{
  int i,j,k;

  k = 0;
  buf[0] = NULL;
  for(i=0;wiseuserroles[i].user != NULL;i++) {
    if(WISEDEBUG) printf("(1) does '%s' = %d '%s'?<BR>\n",ru,i,wiseuserroles[i].user);
    if(strcmp(ru,wiseuserroles[i].user) == 0) {
      for(j=k-1;j >= 0;j--) {
	if(WISEDEBUG) printf("(2) does %d '%s' = %d '%s'?<BR>\n",j,buf[j],i,wiseuserroles[i].project);
	if(strcmp(wiseuserroles[i].project,buf[j]) == 0) {
	  break;
	}
      }
      if(j < 0) {
	buf[k++] = wiseuserroles[i].project;
      }
    }
  }
  buf[k] = NULL;
  return;
}

int
wise_numoffields(char* project)
{
  int i,j;

  for(i=0;wiseproject[i].projectname != NULL;i++) {
    if(strcmp(wiseproject[i].projectname,project)==0) {
      for(j=0;wiseproject[i].fields[j].field != NULL;j++);
      return j;
    }
  }
  return 0;
}

int
wise_getfieldaccess(int startat,char* ru,char* project,char* issueno,int fnum)
{
  int i,j,k;

  for(i=startat;wiseviewaccess[i].project != NULL;i++) {
    if(strcmp(wiseviewaccess[i].project,project)==0) {
      for(k=0;wiseproject[k].projectname != NULL;k++) {
	if(strcmp(wiseproject[k].projectname,project) == 0) {
	  if(strcmp(wiseproject[k].fields[fnum].field,wiseviewaccess[i].field) == 0) {
	    for(j=0;wiseuserroles[j].user != NULL;j++) {
	      if((strcmp(wiseuserroles[j].user,ru) == 0) && (strcmp(wiseuserroles[j].project,project) == 0)) {
		if(strcmp(wiseviewaccess[i].role,wiseuserroles[j].role) == 0) {
		  return i;
		}
	      }
	    }
	  }
	}
      }
    }
  }
  return -1;
}

int
wise_checkcondition(int sock,char* project,char* condition,char* issueno)
{
  char buf[WISEBUFMAX];
  m_result* result;
  m_row row;

  strcpy(buf,"");
  if(condition != NULL && strlen(condition) > 0) {
    sprintf(buf,"select * from %s where %s AND %s = %s",project,condition,"issueno",issueno);
  } else {
    return 1;
  }
  if(WISEDEBUG) printf("condition query = %s\n",buf);
  if(msqlQuery(sock,buf) < 0) {
    printf("<H1>Query error: %s</H1>",msqlErrMsg);
    return 0;
  }
  result = msqlStoreResult();
  if((row = msqlFetchRow(result)) == NULL) {
    msqlFreeResult(result);
    if(WISEDEBUG) printf(" --- NOPE<BR>");
    return 0;
  } else {
    msqlFreeResult(result);
    if(WISEDEBUG) printf(" --- YEP<BR>");
    return 1;
  }
}

void
wise_printfield(int sock,char* ru,char* project,char* issueno,int fnum,int* rnum,m_row row)
{
  int i;
  int access = -1;
  int accessrights;
  char* condition;
  char* values;

  for(i=0;wiseproject[i].projectname != NULL;i++) {
    if(strcmp(wiseproject[i].projectname,project)==0) {
      if(wiseproject[i].fields[fnum].field == NULL) return;
      while((access = wise_getfieldaccess(access+1,ru,project,issueno,fnum)) >= 0) {
	accessrights = wiseviewaccess[access].access;
	condition = wiseviewaccess[access].condition;

        /* call to string macro to replace $USER$ by username */
        condition = wise_stringmacro (condition, ru);
	values = wise_stringmacro(wiseviewaccess[access].values,ru);

	if(wise_checkcondition(sock,project,condition,issueno)) break;
      }
      if(access < 0) {
	accessrights = WISE_R;
	values = "";
      }

      switch(wiseproject[i].fields[fnum].fieldtype) {
      case WISE_TEXT:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>%s<TR>\n",wiseproject[i].fields[fnum].field,accessrights,row[*rnum]);
	  (*rnum)++;
	  break;
	case WISE_RW:
	  printf("<TD>%s<TD>%d<TD><INPUT TYPE=text NAME=%s VALUE=\"%s\" SIZE=%d><TR>\n",
		 wiseproject[i].fields[fnum].field,accessrights,wiseproject[i].fields[fnum].field,row[*rnum],
		 wiseproject[i].fields[fnum].fieldwidth);
	  (*rnum)++;
	  break;
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD><INPUT TYPE=text NAME=%s VALUE=\"%s\" SIZE=%d><TR>\n",
		 wiseproject[i].fields[fnum].field,accessrights,wiseproject[i].fields[fnum].field,values,
		 wiseproject[i].fields[fnum].fieldwidth);
	  (*rnum)++;
	  break;
	case WISE_H:
	  (*rnum)++;
	  break;
	}
	break;
      case WISE_SELECT:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>%s<TR>\n",wiseproject[i].fields[fnum].field,accessrights,row[*rnum]);
	  (*rnum)++;
	  break;
	case WISE_RW:
	  printf("<TD>%s<TD>%d<TD>\n",wiseproject[i].fields[fnum].field,accessrights);
	  printf("<SELECT NAME=%s><OPTION SELECTED>%s\n",wiseproject[i].fields[fnum].field,row[*rnum]);
	  wise_printselectoptions(values);
	  printf("</SELECT><TR>\n");
	  (*rnum)++;
	  break;
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD>\n",wiseproject[i].fields[fnum].field,accessrights);
	  printf("<SELECT NAME=%s>\n",wiseproject[i].fields[fnum].field);
	  wise_printselectoptions(values);
	  printf("</SELECT><TR>\n");
	  (*rnum)++;
	  break;
	case WISE_H:
	  (*rnum)++;
	  break;
	}
	break;
      case WISE_RADIO:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>%s<TR>\n",wiseproject[i].fields[fnum].field,accessrights,row[*rnum]);
	  (*rnum)++;
	  break;
	case WISE_RW:
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD>\n",wiseproject[i].fields[fnum].field,accessrights);
	  wise_printradioitems(wiseproject[i].fields[fnum].field,values,row[*rnum]);
	  printf("<TR>\n");
	  (*rnum)++;
	  break;
	case WISE_H:
	  (*rnum)++;
	  break;
	}
	break;
      case WISE_CBOX:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>%s<TR>\n",wiseproject[i].fields[fnum].field,accessrights,row[*rnum]);
	  (*rnum)++;
	  break;
	case WISE_RW:
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD>\n",wiseproject[i].fields[fnum].field,accessrights);
	  wise_printcheckboxitems(wiseproject[i].fields[fnum].field,values,row[*rnum]);
	  printf("<TR>");
	  (*rnum)++;
	  break;
	case WISE_H:
	  (*rnum)++;
	  break;
	}
	break;
      case WISE_TAREA:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>%s<TR>\n",wiseproject[i].fields[fnum].field,accessrights,row[*rnum]);
	  (*rnum)++;
	  break;
	case WISE_RW:
	  printf("<TD>%s<TD>%d<TD>\n",wiseproject[i].fields[fnum].field,accessrights);
	  printf("<TEXTAREA NAME=%s ROWS=5 COLS=70>%s</TEXTAREA><TR>",wiseproject[i].fields[fnum].field,row[*rnum]);
	  (*rnum)++;
	  break;
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD>\n",wiseproject[i].fields[fnum].field,accessrights);
	  printf("<TEXTAREA NAME=%s ROWS=5 COLS=70></TEXTAREA><TR>",wiseproject[i].fields[fnum].field);
	  (*rnum)++;
	  break;
	case WISE_H:
	  (*rnum)++;
	  break;
	}
	break;
      case WISE_PASSWD:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>%s<TR>\n",wiseproject[i].fields[fnum].field,accessrights,row[*rnum]);
	  (*rnum)++;
	  break;
	case WISE_RW:
	  printf("<TD>%s<TD>%d<TD><INPUT TYPE=password NAME=%s VALUE=\"%s\" SIZE=%d><TR>\n",
		 wiseproject[i].fields[fnum].field,accessrights,wiseproject[i].fields[fnum].field,row[*rnum],
		 wiseproject[i].fields[fnum].fieldwidth);
	  (*rnum)++;
	  break;
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD><INPUT TYPE=password NAME=%s VALUE=\"%s\" SIZE=%d><TR>\n",
		 wiseproject[i].fields[fnum].field,accessrights,wiseproject[i].fields[fnum].field,values,
		 wiseproject[i].fields[fnum].fieldwidth);
	  (*rnum)++;
	  break;
	case WISE_H:
	  (*rnum)++;
	  break;
	}
	break;
      case WISE_INT:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>%s<TR>\n",wiseproject[i].fields[fnum].field,accessrights,row[*rnum]);
	  (*rnum)++;
	  break;
	case WISE_RW:
	  printf("<TD>%s<TD>%d<TD><INPUT TYPE=text NAME=%s VALUE=\"%s\" SIZE=%d><TR>\n",
		 wiseproject[i].fields[fnum].field,accessrights,wiseproject[i].fields[fnum].field,row[*rnum],
		 wiseproject[i].fields[fnum].fieldwidth);
	  (*rnum)++;
	  break;
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD><INPUT TYPE=text NAME=%s VALUE=\"%s\" SIZE=%d><TR>\n",
		 wiseproject[i].fields[fnum].field,accessrights,wiseproject[i].fields[fnum].field,values,
		 wiseproject[i].fields[fnum].fieldwidth);
	  (*rnum)++;
	  break;
	case WISE_H:
	  (*rnum)++;
	  break;
	}
	break;
      case WISE_DATE:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>%s<TR>\n",wiseproject[i].fields[fnum].field,accessrights,row[*rnum]);
	  (*rnum)++;
	  break;
	case WISE_RW:
	  printf("<TD>%s<TD>%d<TD><INPUT TYPE=text NAME=%s VALUE=\"%s\" SIZE=%d><TR>\n",
		 wiseproject[i].fields[fnum].field,accessrights,wiseproject[i].fields[fnum].field,row[*rnum],
		 wiseproject[i].fields[fnum].fieldwidth);
	  (*rnum)++;
	  break;
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD><INPUT TYPE=text NAME=%s VALUE=\"%s\" SIZE=%d><TR>\n",
		 wiseproject[i].fields[fnum].field,accessrights,wiseproject[i].fields[fnum].field,values,
		 wiseproject[i].fields[fnum].fieldwidth);
	  (*rnum)++;
	  break;
	case WISE_H:
	  (*rnum)++;
	  break;
	}
	break;
      case WISE_LOG:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>",wiseproject[i].fields[fnum].field,accessrights);
	  printf("<A HREF=\"http://%s:%s/%s%s-%s-%s\">Click here to see the log</A><TR>\n",WISEHOST,WISEPORT,WISELOGS,issueno,project,wiseproject[i].fields[fnum].field);
	  (*rnum)++;
	  break;
	case WISE_RW:
	  printf("<TD>%s<TD>%d<TD>",wiseproject[i].fields[fnum].field,accessrights);
	  printf("<A HREF=\"http://%s:%s/%s%s-%s-%s\">Click here to see the log</A><BR>\n",WISEHOST,WISEPORT,WISELOGS,issueno,project,wiseproject[i].fields[fnum].field);
	  printf("<TEXTAREA NAME=%s ROWS=5 COLS=70></TEXTAREA><TR>",wiseproject[i].fields[fnum].field);
	  (*rnum)++;
	  break;
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD>",wiseproject[i].fields[fnum].field,accessrights);
	  printf("<TEXTAREA NAME=%s ROWS=5 COLS=70></TEXTAREA><TR>\n",wiseproject[i].fields[fnum].field);
	  (*rnum)++;
	  break;
	case WISE_H:
	  (*rnum)++;
	  break;
	}
	break;
      case WISE_HTML:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>",wiseproject[i].fields[fnum].field,accessrights);
	  printf("<A HREF=\"%s\">\n",row[*rnum+1]);
	  printf("%s</A><TR>",row[*rnum]);
	  (*rnum)++;	(*rnum)++;
	  break;
	case WISE_RW:
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD>",wiseproject[i].fields[fnum].field,accessrights);
	  printf("<A HREF=\"%s\">\n",row[*rnum+1]);
	  printf("%s</A><BR>",row[*rnum]);
	  printf("  New A HREF value = <INPUT TYPE=text NAME=%shtml VALUE=\"%s\"><BR>",wiseproject[i].fields[fnum].field,row[*rnum+1]);
	  printf("  New value = <INPUT TYPE=text NAME=%s VALUE=\"%s\"><TR>\n",wiseproject[i].fields[fnum].field,row[*rnum]);
	  (*rnum)++;	(*rnum)++;
	  break;
	case WISE_H:
	  (*rnum)++;	(*rnum)++;
	  break;
	}
	break;
      }
      break;
    }
  }
}

int
wise_getnewfieldaccess(int startat,char* ru,char* project,char* issueno,int fnum)
{
  int i,j,k;

  for(i=startat;wisenewaccess[i].project != NULL;i++) {
    if(strcmp(wisenewaccess[i].project,project)==0) {
      for(k=0;wiseproject[k].projectname != NULL;k++) {
	if(strcmp(wiseproject[k].projectname,project) == 0) {
	  if(strcmp(wiseproject[k].fields[fnum].field,wisenewaccess[i].field) == 0) {
	    for(j=0;wiseuserroles[j].user != NULL;j++) {
	      if((strcmp(wiseuserroles[j].user,ru) == 0) && (strcmp(wiseuserroles[j].project,project) == 0)) {
		if(wisenewaccess[i].role == NULL || strcmp(wisenewaccess[i].role,wiseuserroles[j].role) == 0) {
		  return i;
		}
	      }
	    }
	  }
	}
      }
    }
  }
  return -1;
}

void
wise_printnewfield(int sock,char* ru,char* project,char* issueno,int fnum)
{
  int i;
  int access = -1;
  int accessrights;
  char* condition;
  char* buf;
  char* values;

  for(i=0;wiseproject[i].projectname != NULL;i++) {
    if(strcmp(wiseproject[i].projectname,project)==0) {
      if(wiseproject[i].fields[fnum].field == NULL) return;
      while((access = wise_getnewfieldaccess(access+1,ru,project,issueno,fnum)) >= 0) {
	accessrights = wisenewaccess[access].access;
	condition = wisenewaccess[access].condition;
	if(wisenewaccess[access].values == NULL) values = "";
	else values = wise_stringmacro(wisenewaccess[access].values,ru);
	if(wise_checkcondition(sock,project,condition,issueno)) break;
      }
      if(access < 0) {
	accessrights = WISE_R;
	values = "";
      }
      switch(wiseproject[i].fields[fnum].fieldtype) {
      case WISE_TEXT:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>%s<TR>",wiseproject[i].fields[fnum].field,accessrights,values);
	  printf("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">",wiseproject[i].fields[fnum].field,values);
	  break;
	case WISE_RW:
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD><INPUT TYPE=text NAME=%s VALUE=\"%s\" SIZE=%d><TR>",
		 wiseproject[i].fields[fnum].field,accessrights,wiseproject[i].fields[fnum].field,values,
		 wiseproject[i].fields[fnum].fieldwidth);
	  break;
	case WISE_H:
	  printf("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">",wiseproject[i].fields[fnum].field,values);
	  break;
	}
	break;
      case WISE_SELECT:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>%s<TR>",wiseproject[i].fields[fnum].field,accessrights,values);
	  printf("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">",wiseproject[i].fields[fnum].field,values);
	  break;
	case WISE_RW:
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD>",wiseproject[i].fields[fnum].field,accessrights);
	  printf("<SELECT NAME=%s>",wiseproject[i].fields[fnum].field);
	  wise_printselectoptions(values);
	  printf("</SELECT><TR>");
	  break;
	case WISE_H:
	  printf("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">",wiseproject[i].fields[fnum].field,values);
	  break;
	}
	break;
      case WISE_RADIO:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>%s<TR>",wiseproject[i].fields[fnum].field,accessrights,values);
	  printf("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">",wiseproject[i].fields[fnum].field,values);
	  break;
	case WISE_RW:
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD>",wiseproject[i].fields[fnum].field,accessrights);
	  wise_printradioitems(wiseproject[i].fields[fnum].field,values,NULL);
	  printf("<TR>");
	  break;
	case WISE_H:
	  printf("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">",wiseproject[i].fields[fnum].field,values);
	  break;
	}
	break;
      case WISE_CBOX:
	switch(accessrights) {
	case WISE_R:
	  if(atoi(values)) {
	    printf("<TD>%s<TD>%d<TD>YES<TR>",wiseproject[i].fields[fnum].field,accessrights);
	  } else {
	    printf("<TD>%s<TD>%d<TD>NO<TR>",wiseproject[i].fields[fnum].field,accessrights);
	  }
	  printf("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">",wiseproject[i].fields[fnum].field,values);
	  break;
	case WISE_RW:
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD>",wiseproject[i].fields[fnum].field,accessrights);
	  wise_printcheckboxitems(wiseproject[i].fields[fnum].field,values,NULL);
	  printf("<TR>");
	  break;
	case WISE_H:
	  printf("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">",wiseproject[i].fields[fnum].field,values);
	  break;
	}
	break;
      case WISE_TAREA:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>%s<TR>",wiseproject[i].fields[fnum].field,accessrights,values);
	  printf("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">",wiseproject[i].fields[fnum].field,values);
	  break;
	case WISE_RW:
	  printf("<TD>%s<TD>%d<TD>",wiseproject[i].fields[fnum].field,accessrights);
	  printf("<TEXTAREA NAME=%s ROWS=5 COLS=70>%s</TEXTAREA><TR>",wiseproject[i].fields[fnum].field,values);
	  break;
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD>",wiseproject[i].fields[fnum].field,accessrights);
	  printf("<TEXTAREA NAME=%s ROWS=5 COLS=70></TEXTAREA><TR>",wiseproject[i].fields[fnum].field);
	  break;
	case WISE_H:
	  printf("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">",wiseproject[i].fields[fnum].field,values);
	  break;
	}
	break;
      case WISE_PASSWD:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>%s<TR>",wiseproject[i].fields[fnum].field,accessrights,values);
	  printf("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">",wiseproject[i].fields[fnum].field,values);
	  break;
	case WISE_RW:
	  printf("<TD>%s<TD>%d<TD><INPUT TYPE=password NAME=%s VALUE=\"%s\" SIZE=%d><TR>",
		 wiseproject[i].fields[fnum].field,accessrights,wiseproject[i].fields[fnum].field,values,
		 wiseproject[i].fields[fnum].fieldwidth);
	  break;
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD><INPUT TYPE=password NAME=%s VALUE=\"%s\" SIZE=%d><TR>",
		 wiseproject[i].fields[fnum].field,accessrights,wiseproject[i].fields[fnum].field,values,
		 wiseproject[i].fields[fnum].fieldwidth);
	  break;
	case WISE_H:
	  printf("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">",wiseproject[i].fields[fnum].field,values);
	  break;
	}
	break;
      case WISE_INT:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>%s<TR>",wiseproject[i].fields[fnum].field,accessrights,values);
	  printf("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">",wiseproject[i].fields[fnum].field,values);
	  break;
	case WISE_RW:
	  printf("<TD>%s<TD>%d<TD><INPUT TYPE=text NAME=%s VALUE=\"%s\" SIZE=%d><TR>",
		 wiseproject[i].fields[fnum].field,accessrights,wiseproject[i].fields[fnum].field,values,
		 wiseproject[i].fields[fnum].fieldwidth);
	  break;
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD><INPUT TYPE=text NAME=%s VALUE=\"%s\" SIZE=%d><TR>",
		 wiseproject[i].fields[fnum].field,accessrights,wiseproject[i].fields[fnum].field,values,
		 wiseproject[i].fields[fnum].fieldwidth);
	  break;
	case WISE_H:
	  printf("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">",wiseproject[i].fields[fnum].field,values);
	  break;
	}
	break;
      case WISE_DATE:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>%s<TR>",wiseproject[i].fields[fnum].field,accessrights,values);
	  printf("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">",wiseproject[i].fields[fnum].field,values);
	  break;
	case WISE_RW:
	  printf("<TD>%s<TD>%d<TD><INPUT TYPE=text NAME=%s VALUE=\"%s\" SIZE=%d><TR>",
		 wiseproject[i].fields[fnum].field,accessrights,wiseproject[i].fields[fnum].field,values,
		 wiseproject[i].fields[fnum].fieldwidth);
	  break;
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD><INPUT TYPE=text NAME=%s VALUE=\"%s\" SIZE=%d><TR>",
		 wiseproject[i].fields[fnum].field,accessrights,wiseproject[i].fields[fnum].field,values,
		 wiseproject[i].fields[fnum].fieldwidth);
	  break;
	case WISE_H:
	  printf("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">",wiseproject[i].fields[fnum].field,values);
	  break;
	}
	break;
      case WISE_LOG:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>",wiseproject[i].fields[fnum].field,accessrights);
	  printf("<A HREF=\"http://%s:%s/%s%s-%s-%s\">Click here to see the log</A><TR>",WISEHOST,WISEPORT,WISELOGS,issueno,project,wiseproject[i].fields[fnum].field);
	  break;
	case WISE_RW:
	  printf("<TD>%s<TD>%d<TD>",wiseproject[i].fields[fnum].field,accessrights);
	  printf("<A HREF=\"http://%s:%s/%s%s-%s-%s\">Click here to see the log</A><BR>",WISEHOST,WISEPORT,WISELOGS,issueno,project,wiseproject[i].fields[fnum].field);
	  printf("<TEXTAREA NAME=%s ROWS=5 COLS=70></TEXTAREA><TR>",wiseproject[i].fields[fnum].field);
	  break;
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD>",wiseproject[i].fields[fnum].field,accessrights);
	  printf("<TEXTAREA NAME=%s ROWS=5 COLS=70></TEXTAREA><TR>",wiseproject[i].fields[fnum].field);
	  break;
	case WISE_H:
	  break;
	}
	break;
      case WISE_HTML:
	switch(accessrights) {
	case WISE_R:
	  printf("<TD>%s<TD>%d<TD>",wiseproject[i].fields[fnum].field,accessrights);
	  wise_printhtml(values,ru);
	  wise_printhtmlfields(wiseproject[i].fields[fnum].field,values,ru);
	  printf("<TR>");
	  break;
	case WISE_RW:
	  printf("<TD>%s<TD>%d<TD>",wiseproject[i].fields[fnum].field,accessrights);
	  wise_printhtml(values,ru);
	  printf("  New A HREF value = <INPUT TYPE=text NAME=%shtml SIZE=%d VALUE=\"",
		 wiseproject[i].fields[fnum].field,wiseproject[i].fields[fnum].fieldwidth);
	  wise_printhtmlfield(values,2);
	  printf("\"><BR>");
	  printf("  New value = <INPUT TYPE=text NAME=%s SIZE=%d VALUE=\"",
		 wiseproject[i].fields[fnum].field,wiseproject[i].fields[fnum].fieldwidth);
	  wise_printhtmlfield(values,1);
	  printf("\">");
	  printf("<TR>");
	  break;
	case WISE_W:
	  printf("<TD>%s<TD>%d<TD>",wiseproject[i].fields[fnum].field,accessrights);
	  printf("  New A HREF value = <INPUT TYPE=text NAME=%shtml SIZE=%d VALUE=\"http://\"><BR>",wiseproject[i].fields[fnum].field);
	  printf("  New value = <INPUT TYPE=text NAME=%s SIZE=%d>",wiseproject[i].fields[fnum].field);
	  printf("<TR>");
	  break;
	case WISE_H:
	  wise_printhtmlfields(wiseproject[i].fields[fnum].field,values,ru);
	  break;
	}
	break;
      }
      break;
    }
  }
}

void
wise_printactions(char* ru,char* issueno,char* project)
{
  int j;

  for(j=0;wiseviewactions[j].project != NULL;j++) {
    if(strcmp(project,wiseviewactions[j].project) == 0) {
      if(wise_playsrole(ru,wiseviewactions[j].role,project)) {
	printf("<INPUT TYPE=submit NAME=\"%s\" VALUE=\"%s\">",wiseviewactions[j].program,wiseviewactions[j].label);
      }
    }
  }
  printf("<INPUT TYPE=reset NAME=\"reset\" VALUE=\"Reset All Values\">");
}

int
wise_deleteissue(char* project,char* issueno)
{
  int sock;
  char tbuf[WISEBUFMAX];
  char* db;
  char* dbhost;

  if((dbhost = wise_getprojhost(project)) == NULL) {
    printf("<H1>Could not find project %s database host!</H1>",project);
    return;
  }
  if((sock = msqlConnect(dbhost)) < 0) {
    printf("<H1>Msql Error: %s</H1>",msqlErrMsg);
    return 0;
  }
  if((db = wise_getprojdb(project)) == NULL) {
    printf("<H1>Could not get project's database name for deletion action!</H1>");
    return 0;
  }
  if(msqlSelectDB(sock,db) < 0) {
    printf("<H1>Select error: %s</H1>",msqlErrMsg);
    return 0;
  }

  sprintf(tbuf,"delete from %s where issueno = %s",project,issueno);

  if(msqlQuery(sock,tbuf) < 0) {
    printf("<BR><BR>Delete key %s fail: %s<BR>",issueno,tbuf);
    return 0;
  }

  msqlClose(sock);
  return 1;
}

void
wise_printnewactions(char* ru)
{
  printf("<INPUT TYPE=submit NAME=\"update\" VALUE=\"Submit New Form\">");
  printf("<INPUT TYPE=reset NAME=\"reset\" VALUE=\"Reset All Values\">");
}

int
wise_newissue(int sock,char* project)
{
  int i,j;
  int attempts = 0;
  int issuenum = 0;
  m_result* result;
  m_row row;
  char buf[WISEBUFMAX];
  char tbuf[WISEBUFMAX];

  sprintf(buf,"select intval from %sdata where propname = 'MAXISSUENO'",
	  project);
  if(WISEDEBUG) printf("max query = %s<BR>\n",buf);
  if(msqlQuery(sock,buf) < 0) {
    printf("<H1>Query error: %s</H1>",msqlErrMsg);
    return -1;
  }
  if(WISEDEBUG) printf("max query = %s<BR>\n",buf);
  result = msqlStoreResult();
  if((row = msqlFetchRow(result)) == NULL) {
    printf("<H1>Fetch error: %s</H1>",msqlErrMsg);
    return -1;
  }
  if(WISEDEBUG) printf("max query = %s<BR>\n",buf);
  issuenum = atoi(row[0])+1;
  sprintf(buf,"insert into %s (issueno,project",project);
  for(i=0;wiseproject[i].projectname != NULL;i++) {
    if(strcmp(wiseproject[i].projectname,project)==0) {
      for(j=0;wiseproject[i].fields[j].field != NULL;j++) {
	strcat(buf,",");
	strcat(buf,wiseproject[i].fields[j].field);
	switch(wiseproject[i].fields[j].fieldtype) {
	case WISE_TEXT:
	  break;
	case WISE_SELECT:
	  break;
	case WISE_TAREA:
	  break;
	case WISE_HTML:
	  strcat(buf,",");
	  strcat(buf,wiseproject[i].fields[j].field);
	  strcat(buf,"html");
	  break;
	case WISE_RADIO:
	  break;
	case WISE_DATE:
	  break;
	case WISE_LOG:
	  break;
	case WISE_INT:
	  break;
	case WISE_CBOX:
	  break;
	case WISE_PASSWD:
	  break;
	}
      }
      strcat(buf,") values (%d,'BLANK'");
      for(j=0;wiseproject[i].fields[j].field != NULL;j++) {
	strcat(buf,",");
	switch(wiseproject[i].fields[j].fieldtype) {
	case WISE_TEXT:
	  strcat(buf,"''");
	  break;
	case WISE_SELECT:
	  strcat(buf,"''");
	  break;
	case WISE_TAREA:
	  strcat(buf,"''");
	  break;
	case WISE_HTML:
	  strcat(buf,"'',''");
	  break;
	case WISE_RADIO:
	  strcat(buf,"''");
	  break;
	case WISE_DATE:
	  strcat(buf,"''");
	  break;
	case WISE_LOG:
	  strcat(buf,"0");
	  break;
	case WISE_INT:
	  strcat(buf,"0");
	  break;
	case WISE_CBOX:
	  strcat(buf,"''");
	  break;
	case WISE_PASSWD:
	  strcat(buf,"''");
	  break;
	}
      }
      break;
    }
  }
  strcat(buf,")");
  if(WISEDEBUG) printf("maxquery = %s<BR>\n",buf);
  sprintf(tbuf,buf,issuenum);
  while(msqlQuery(sock,tbuf) < 0 && attempts++ < WISEMAXATT) {
    if(WISEDEBUG) printf("Failed %d, trying %s<BR>",issuenum++,issuenum);
    sprintf(tbuf,buf,issuenum);
  }
  if(attempts >= WISEMAXATT) {
    printf("<H1>New Issue Error: Could not allocate new issue number</H1>");
    return -1;
  } else {
    sprintf(buf,"update %sdata set intval = %d where propname = 'MAXISSUENO'",project,issuenum);
    if(msqlQuery(sock,buf) < 0) {
      printf("<H1>New Issue Error: Could not set max issue number</H1>");
      return -1;
    }
    return issuenum;
  }
}

int
wise_playsrole(char* ru,char* role,char* project)
{
  int i;

  for(i=0;wiseuserroles[i].user != NULL;i++) {
    if((strcmp(ru,wiseuserroles[i].user) == 0)
       && (role == NULL || (strcmp(role,wiseuserroles[i].role) == 0))
       && (project == NULL || (strcmp(project,wiseuserroles[i].project) == 0))) return 1;
  }
  return 0;
}	

void
wise_printtdlactions(char* ru,char* project,char* extra, form_entry *parms) 
{
  int j;

  for(j=0;wisetdlactions[j].project != NULL;j++) {
    if(strcmp(project,wisetdlactions[j].project) == 0 && wise_playsrole(ru,wisetdlactions[j].role,wisetdlactions[j].project)) {
      printf("<A HREF=\"http://%s:%s/%s?project=%s%s",
	     WISEHOST,WISEPORT,wisetdlactions[j].program,wisetdlactions[j].project,extra);
#ifdef NEWREPORT
      wise_print_query_opts (project, parms); 
#endif /* NEWREPORT */
      printf("\"><B>%s</B></A> | ", wisetdlactions[j].label);
    }
  }
}

void
wise_formatupdate(char* buf,char* project,form_entry* p,char* issueno,char* ru)
{
  form_entry* tp;
  int i,j;

  if(strstr(p->name,"html") != NULL) {
    sprintf(buf,"%s='%s'",p->name,p->val);
    return;
  }
  for(i=0;wiseproject[i].projectname != NULL;i++) {
    if(strcmp(wiseproject[i].projectname,project) == 0) {
      for(j=0;wiseproject[i].fields[j].field != NULL;j++) {
	if(strcmp(wiseproject[i].fields[j].field,p->name) == 0) {
	  switch(wiseproject[i].fields[j].fieldtype) {
	  case WISE_TEXT:
	    sprintf(buf,"%s='%s'",p->name,p->val);
	    break;
	  case WISE_SELECT:
	    sprintf(buf,"%s='%s'",p->name,p->val);
	    break;
	  case WISE_TAREA:
	    sprintf(buf,"%s='%s'",p->name,p->val);
	    break;
	  case WISE_HTML:
	    sprintf(buf,"%s='%s'",p->name,p->val);
	    break;
	  case WISE_RADIO:
	    sprintf(buf,"%s='%s'",p->name,p->val);
	    break;
	  case WISE_DATE:
	    sprintf(buf,"%s='%s'",p->name,p->val);
	    break;
	  case WISE_LOG:
	    wise_appendlog(p->name,p->val,project,issueno,ru);
	    break;
	  case WISE_INT:
	    sprintf(buf,"%s=%d",p->name,atoi(p->val));
	    break;
	  case WISE_CBOX:
	    strcpy(buf,p->name);
	    strcat(buf,"='");
	    strcat(buf,p->val);
	    for(tp=p->next;tp != NULL;tp=tp->next) {
	      if(strcmp(tp->name,p->name) == 0) {
		strcat(buf,"&");
		strcat(buf,tp->val);
		tp->name = "";
	      }
	    }
	    strcat(buf,"'");
	    break;
	  default:
	    sprintf(buf,"%s='%s'",p->name,p->val);
	    break;
	  }
	}
      }
    }
  }
}

void
wise_formatlogupdate(char* names,char* values,char* project,form_entry* p)
{
  form_entry *tp;
  int i,j;

  if(strstr(p->name,"html") != NULL) {
    strcat(names,p->name);
    strcat(values,"'");
    strcat(values,p->val);
    strcat(values,"'");
    return;
  } 
  for(i=0;wiseproject[i].projectname != NULL;i++) {
    if(strcmp(wiseproject[i].projectname,project) == 0) {
      for(j=0;wiseproject[i].fields[j].field != NULL;j++) {
	if(strcmp(wiseproject[i].fields[j].field,p->name) == 0) {
	  switch(wiseproject[i].fields[j].fieldtype) {
	  case WISE_TEXT:
	    strcat(names,p->name);
	    sprintf(values,"'%s'",p->val);
	    break;
	  case WISE_SELECT:
	    strcat(names,p->name);
	    sprintf(values,"'%s'",p->val);
	    break;
	  case WISE_TAREA:
	    strcat(names,p->name);
	    sprintf(values,"'%s'",p->val);
	    break;
	  case WISE_HTML:
	    strcat(names,p->name);
	    sprintf(values,"'%s'",p->val);
	    break;
	  case WISE_RADIO:
	    strcat(names,p->name);
	    sprintf(values,"'%s'",p->val);
	    break;
	  case WISE_DATE:
	    strcat(names,p->name);
	    sprintf(values,"'%s'",p->val);
	    break;
	  case WISE_LOG:
	    strcat(names,p->name);
	    strcat(values,"0");
	    break;
	  case WISE_INT:
	    strcat(names,p->name);
	    sprintf(values,"%d",atoi(p->val));
	    break;
	  case WISE_CBOX:
	    strcat(names,p->name);
	    strcat(values,"'");
	    strcat(values,p->val);
	    for(tp=p->next;tp != NULL;tp=tp->next) {
	      if(strcmp(tp->name,p->name) == 0) {
		strcat(values,"&");
		strcat(values,tp->val);
		tp->name = "";
	      }
	    }
	    strcat(values,"'");
	    break;
	  default:
	    strcat(names,p->name);
	    sprintf(values,"'%s'",p->val);
	    break;
	  }
	}
      }
    }
  }
}

int
wise_appendlog(char* name,char* val,char* project,char* issueno,char* ru)
{
  char filename[WISEBUFMAX];
  char dbuf[WISEBUFMAX];
  time_t ts1970;
  struct tm *pstm;
  FILE *f;

  sprintf(filename,"%s%s-%s-%s",WISEPATHLOG,issueno,project,name);
  if((f = fopen(filename,"a+")) == NULL) {
    return -1;
  }
  fprintf(f,"\n========================================\n");
  ts1970 = time(NULL);
  /* cftime(dbuf,"%Y-%m-%d %T",&ts1970);*/
  pstm = localtime (&ts1970);
  strftime(dbuf,WISEBUFMAX,"%Y-%m-%d %T",pstm);
  fprintf(f,"%s %s\n",ru,dbuf);
  fprintf(f,"========================================\n");
  fprintf(f,val);
  fclose(f);
  return 0;
}


#ifdef NEWREPORT
void wise_print_hidden_opts (char * projname, form_entry *parms) 
{
  form_entry *p;
  int len = strlen (projname);

  /* print hidden only if doesnt match */
  for(p=parms;p;p=p->next) {
    if (!strncmp (projname, p->name, len)) {
      printf ("<INPUT TYPE=hidden NAME=%s VALUE=\"%s\">\n", p->name, p->val);
    }
  }
}


/* assemble a cgi query string out of selected parameters */
void wise_print_query_opts (char * projname, form_entry *parms) 
{
  form_entry *p;
  int len = strlen (projname);

  /* print only those tokens that have the correct name */
  for(p=parms;p;p=p->next) {
    if (!strncmp (projname, p->name, len)) {
      char tmpbuf [WISEBUFMAX];
      sanitize (tmpbuf, p->val);
      printf ("&%s=%s", p->name, tmpbuf);
    }
  }
}

#endif /* NEWREPORT */

